/*
    TIHex v0.41
    Copyright (C) 2004 Jonas Gehring
    This programm uses functions from ExtGraph v2.00beta2 and TIChess by TIChessTeam
        
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 10.04.04; 14:18:13

#include <tigcclib.h>					// Include all standard header files
#include "tihex.h"						// Include "tihex"-header
#include "extgraph.h"					// Include ExtGraph v2.00beta2 by TIChessTeam

// Global variables: The game field, player number, positions
int gamefield[10][10];
int player = PLAYER_1;
// These variables are signed because the can go until -1 before being checked 
signed int xpos = 0, ypos = 5;
// The temporary positions for the undo-function
int xtemp = 0, ytemp = 0;
int undo_action = 1;

// The 'main' routine of this C Source file
int _startgame(void)
{
	int action;
	
	// The standard settings
	_clearfield();
	_nsprite(player);
	xpos = 0, ypos = 5;
	_drawcoin(player, xpos, ypos, DRAW);
	
	gameplay:
	_redraw();
	action = 0;
	undo_action = 1;
	
	// As long as no one wants to quit, do...
	while (action != EXIT)
	{
		// Draw the arrows
		if (player == PLAYER_1)
			GraySprite16_OR(60, 20, CURS_HEIGHT, curs1light, curs1dark, GetPlane(LIGHT_PLANE), GetPlane(DARK_PLANE));
		else
			GraySprite16_AND(60, 85, CURS_HEIGHT, curs2light, curs2dark, GetPlane(LIGHT_PLANE), GetPlane(DARK_PLANE));

		action = _keywait();
		// If someone has won, exit the game
		if (action == PLAYER1_HAS_WON || action == PLAYER2_HAS_WON)
			goto exit;
		
		// If a new game is selected, do the standard settings
		if (action == NEW_GAME)
		{
			xpos = 0, ypos = 5;
			_clearfield();
		}
		
		// Undo-function
		if (action == UNDO && !undo_action)
			_undo();
		
	}	
	
	// If s.o. wants to quit, show this message
	_drawrect(40);
	DrawGrayStrExt(0, 65, "Do you really want to quit?", A_REVERSE | A_CENTERED, SMALL);
	if (ngetchx() == KEY_ESC)
		 goto gameplay;
	
	exit:
	return action;	
}


// Move the cursor right
void _moveright(void)
{
	_checkfield(1);
	xpos++;
	if (xpos == 10)
		xpos = 0;
	_checkfield(2);	
}

// Move the cursor left
void _moveleft(void)
{
	_checkfield(1);
	xpos--;
	if (xpos == -1)
		xpos = 9;
	_checkfield(2);	;	
}

// Move the cursor up
void _moveup(void)
{
	_checkfield(1);
	ypos++;
	if (ypos == 10)
		ypos = 0;
	_checkfield(2);	
}

// Move the cursor down
void _movedown(void)
{
	_checkfield(1);
	ypos--;
	if (ypos == -1)
		ypos = 9;
	_checkfield(2);		
}

// Checks if a field is blank and reacts if not
void _checkfield(int mode)
{
	if (mode == 1)
	{
		if (gamefield[xpos][ypos] == 0)
			_drawcoin(player, xpos, ypos, CLEAR);
		else
		{
			if (gamefield[xpos][ypos] == 1)
				_drawcoin(PLAYER_1, xpos, ypos, DRAW);
			else
				_drawcoin(PLAYER_2, xpos, ypos, DRAW);		
		}
	}
	else
	{
		if (gamefield[xpos][ypos] != 0)
			_drawcoin(player, xpos, ypos, FULL);
		else
			_drawcoin(player, xpos, ypos, DRAW);
	}
}


// Undo funciotn
void _undo(void)
{
	int oldplayer;
	
	_drawcoin(player, xtemp, ytemp, CLEAR);
	
	oldplayer = player;
	
	if (player == PLAYER_1)
		player = PLAYER_2;
	else
		player = PLAYER_1;
	
	// Reset position in the 'matrix'
  gamefield[xtemp][ytemp] = 0;
  _redraw();
 
  if (gamefield[xpos][ypos])
  	_drawcoin(oldplayer, xpos, ypos, DRAW);		  
  else
  	_drawcoin(oldplayer, xpos, ypos, CLEAR);
  
  xpos = xtemp, ypos = ytemp; 
  _drawcoin(player, xpos, ypos, DRAW);
  
  undo_action = 1;
}



// Sets a field
int _setcoin(void)
{
	int act;
	
	// Check if the hex is empty
	// If not, return without doing anything
	if (gamefield[xpos][ypos] == 0)
		gamefield[xpos][ypos] = player;
	else
		return 0;
	
	// Check if s.o. has won
	act = _proof();
	if (act == 1)
	{
		if (player == PLAYER_1)
			return PLAYER1_HAS_WON;
		else
			return PLAYER2_HAS_WON;		
	}
	
	// Store the old values for xpos and ypos
	// in xtemp and ytemp to enable undoing
	xtemp = xpos, ytemp = ypos;
	
	// It's the other player's turn!
	if (player == PLAYER_1)
	{
		xpos = 5; ypos = 0;
		player = PLAYER_2;
	}
	else
	{
		xpos = 0; ypos = 5;
		player = PLAYER_1;
  }
  
  // Erase the arrows
  DrawGrayRect(60, 20, 76, 36, COLOR_LIGHTGRAY, RECT_FILLED);
  DrawGrayRect(60, 85, 76, 101, COLOR_LIGHTGRAY, RECT_FILLED);
  
  // Draw player sprite and a "cursor"
  _nsprite(player);
  _checkfield(2);
	
	// We can undo our acion now
	undo_action = 0;
		
	return 0;
}


// Redraws the field
void _redraw(void)
{
	int s,i;
	
	// Decompress the backround picture
	DecompRLEVert(field2, GetPlane(LIGHT_PLANE), 7680/2);	
	DecompRLEVert(field1, GetPlane(DARK_PLANE), 7680/2);
	FastDrawGrayLine(0, 0, 50, 0, COLOR_LIGHTGRAY);
	
	// Redraw every hex
	for (s = 0; s <= 9; s++)
	{
		for (i = 0; i <= 9; i++)
		{
			if (gamefield[s][i] != 0)
				_drawcoin(gamefield[s][i], s, i, DRAW); 
		}
	}
	
	// Draw player sprite and draw "cursor"
	_nsprite(player);
	_checkfield(2);
}


// Clears the field
void _clearfield(void)
{
	// Set the gamefield to NULL everywhere
	memset (gamefield, 0, 10*10*sizeof(int));
	
	// It's player one's turn!
	player = PLAYER_1;
	_redraw();	
}


// The following comes from David Randall (known for International Karate+)
// Thanks for his great help!!

// the global temporary field variable is used to store if a Hex is being
// checked.
int tempfield[10][10];

int _CheckHexPlayer1(int s, int i)
{
 // have we made to the other side?
 if (s > 9)
  return 1;

 // ensure this is a valid Hex
 if (s < 0 || i < 0 || i > 9)
  return 0;

 // does Player 1 own this Hex?
 if (gamefield[s][i] != PLAYER_1)
  return 0;

 // has this Hex already been checked?
 if (tempfield[s][i] == 1)
  return 0;

 // mark this hex as currently being checked so we don't keep checking
 // it
 tempfield[s][i] = 1;

 // check each hex around this hex
 if (_CheckHexPlayer1(s+0,i-1)) return 1;
 if (_CheckHexPlayer1(s+1,i-1)) return 1;
 if (_CheckHexPlayer1(s-1,i+0)) return 1;
 if (_CheckHexPlayer1(s+1,i+0)) return 1;
 if (_CheckHexPlayer1(s-1,i+1)) return 1;
 if (_CheckHexPlayer1(s+0,i+1)) return 1;

 // this Hex does not have anything around it
 return 0;
}

int _CheckHexPlayer2(int s, int i)
{
 // have we made to the other side?
 if (i > 9)
  return 1;

 // ensure this is a valid Hex
 if (i < 0 || s < 0 || s > 9)
  return 0;

 // does Player 2 own this Hex?
 if (gamefield[s][i] != PLAYER_2)
  return 0;

 // has this Hex already been checked?
 if (tempfield[s][i] == 1)
  return 0;

 // mark this hex as currently being checked so we don't keep checking
 // it
 tempfield[s][i] = 1;

 // check each hex around this hex
 if (_CheckHexPlayer2(s+0,i-1)) return 1;
 if (_CheckHexPlayer2(s+1,i-1)) return 1;
 if (_CheckHexPlayer2(s-1,i+0)) return 1;
 if (_CheckHexPlayer2(s+1,i+0)) return 1;
 if (_CheckHexPlayer2(s-1,i+1)) return 1;
 if (_CheckHexPlayer2(s+0,i+1)) return 1;

 // this Hex does not have anything around it
 return 0;
}

int _proof()
{
 int s,i;
 int pathFound;

 // Clear the temporary field for Player 1
 memset (tempfield, 0, 10*10*sizeof(int));

 // Check to see if Player 1 has finished
 for (i = 0; i < 10; i++)
 {
  // does Player 1 own this Hex?
  if (gamefield[0][i] == PLAYER_1)
  {
   // Search for a complete path from Start to End
   pathFound = _CheckHexPlayer1(0, i);

   if (pathFound)
   {
    // Player 1 has won
    return 1;
   }
  }
 }

 // Clear the temporary field for Player 2
 memset (tempfield, 0, 10*10*sizeof(int));

 // Check to see if Player 2 has finished
 for (s = 0; s < 10; s++)
 {
  // does Player 2 own this Hex?
  if (gamefield[s][0] == PLAYER_2)
  {
   // Search for a complete path from Start to End
   pathFound = _CheckHexPlayer2(s, 0);

   if (pathFound)
   {
    // Player 2 has won
    return 1;
   }
  }
 }

 return 0;
}
